@@ -65,9 +65,6 @@ sub run
     print "Using config file: $self->{cfg}->{configfile}\n\n"
         if ($r && !$self->{cfg}->{silent});
 
-    $self->init_words()       # Init words. (Foulwords, ignorewords, etc.)
-        if ($self->{use_configfile});
-
     # Get translations from langfile
     $self->get_language_templates();
 
@@ -135,6 +132,7 @@ sub get_default_config_settings
         # Stats settings
 
         showactivetimes => 1,
+        showactivenicks => 1,
         showbignumbers => 1,
         showtopics => 1,
         showlinetime => 0,
@@ -148,6 +146,7 @@ sub get_default_config_settings
         showkickline => 1,
         showactionline => 1,
         showfoulline => 0,
+        showfouldecimals => 1,
         showshoutline => 1,
         showviolentlines => 1,
         showrandquote => 1,
@@ -168,6 +167,7 @@ sub get_default_config_settings
         minquote => 25,
         maxquote => 65,
         quotewidth => 80,
+        bignumbersthreshold => 'sqrt',
         wordlength => 5,
         activenicks => 25,
         activenicks2 => 30,
@@ -199,7 +199,7 @@ sub get_default_config_settings
         botnicks => '',            # Needed for DCpp format (non-irc)
 
         dailyactivity => 0,
-        version => "0.56",
+        version => "0.62",
     };
 
     # This enables us to use the search_path in other modules
@@ -244,14 +244,6 @@ sub get_language_templates
     close(FILE);
 }
 
-sub init_words
-{
-    my $self = shift;
-    $self->{cfg}->{foulwords} = wordlist_regexp($self->{cfg}->{foulwords}, $self->{cfg}->{regexpaliases});
-    $self->{cfg}->{ignorewords} = wordlist_regexp($self->{cfg}->{ignorewords}, $self->{cfg}->{regexpaliases});
-    $self->{cfg}->{violentwords} = wordlist_regexp($self->{cfg}->{violentwords}, $self->{cfg}->{regexpaliases});
-}
-
 sub init_config
 {
     my $self = shift;
@@ -409,6 +401,11 @@ sub init_pisg
     }
     $self->{cfg}->{timestamp} = $timestamp;
 
+    # convert wordlists
+    $self->{cfg}->{foulwords} = wordlist_regexp($self->{cfg}->{foulwords}, $self->{cfg}->{regexpaliases});
+    $self->{cfg}->{ignorewords} = wordlist_regexp($self->{cfg}->{ignorewords}, $self->{cfg}->{regexpaliases});
+    $self->{cfg}->{violentwords} = wordlist_regexp($self->{cfg}->{violentwords}, $self->{cfg}->{regexpaliases});
+
     # Add trailing slash when it's not there..
     $self->{cfg}->{imagepath} =~ s/([^\/])$/$1\//;
     # Set ImageGlobPath default
@@ -512,7 +509,7 @@ sub parse_channels
             print STDERR "Channel $channel not in config file, ignoring\n";
             next;
         }
-        foreach (keys %{ $self->{chans}->{$channel} }) {
+        foreach (keys %{ $self->{chans}->{$channel} }) { # import channel specific config
             $self->{cfg}->{$_} = $self->{chans}->{$channel}{$_};
         }
         $self->do_channel();
