@@ -33,7 +33,8 @@ sub new
             # convert from template charset to our
             $self->{iconv} = Text::Iconv->new($lang_charset, $self->{cfg}->{charset});
         } else {
-            print STDERR "Text::Iconv is not available, skipping charset conversion for language templates\n";
+            print "Text::Iconv is not installed, skipping charset conversion of language templates\n"
+                unless ($self->{cfg}->{silent});
         }
     }
 
@@ -52,9 +53,13 @@ sub create_output
 
     $self->_topactive();
 
+    if($self->{cfg}->{bignumbersthreshold} =~ /^sqrt/) {
+        $self->{cfg}->{bignumbersthreshold} = int(sqrt($self->{stats}->{topactive_lines}));
+    }
+
     my $fname = $self->{cfg}->{outputfile};
     $fname =~ s/\%t/$self->{cfg}->{outputtag}/g;
-    print "Now generating HTML($fname)...\n"
+    print "Now generating HTML in $fname...\n"
         unless ($self->{cfg}->{silent});
 
     open (OUTPUT, "> $fname") or
@@ -93,7 +98,10 @@ sub create_output
     if ($self->{cfg}->{showactivetimes}) {
         $self->_activetimes();
     }
-    $self->_activenicks();
+
+    if ($self->{cfg}->{showactivenicks}) {
+        $self->_activenicks();
+    }
 
     if ($self->{cfg}->{showmostactivebyhour}) {
         $self->_mostactivebyhour();
@@ -175,26 +183,33 @@ sub _htmlheader
         nicks      => scalar keys %{ $self->{stats}->{lines} }
     );
 
-    my $css_file = $self->{cfg}->{cssdir} . $self->{cfg}->{colorscheme} . ".css";
-
-    # Get the chosen CSS file
-    open(FILE, $css_file) or open (FILE, $self->{cfg}->{search_path} . "/$css_file") or die("$0: Unable to open stylesheet($css_file): $!\n");
-
-    my @CSS = <FILE>;
+    my $CSS;
+    if($self->{cfg}->{colorscheme} =~ /[^\w]/) { # use external CSS file
+        $CSS = "<link rel=\"stylesheet\" type=\"text/css\" href=\"$self->{cfg}->{colorscheme}\">";
+    } elsif($self->{cfg}->{colorscheme} ne "none") { # read the chosen CSS file
+        my $css_file = $self->{cfg}->{cssdir} . $self->{cfg}->{colorscheme} . ".css";
+        open(FILE, $css_file) or open (FILE, $self->{cfg}->{search_path} . "/$css_file") or die("$0: Unable to open stylesheet $css_file: $!\n");
+        {
+            local $/; # enable "slurp" mode
+            $CSS = "<style type=\"text/css\">\n". <FILE>. "</style>";
+        }
+        close FILE;
+    }
 
     my $title = $self->_template_text('pagetitle1', %hash);
-    print OUTPUT <<HTML;
+    if($self->{cfg}->{colorscheme} ne "none") {
+        print OUTPUT <<HTML;
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html>
 <head>
 <meta http-equiv="Content-Type" content="text/html; charset=$self->{cfg}->{charset}" />
 <title>$title</title>
-<style type="text/css">
-@CSS
-</style></head>
+$CSS
+</head>
 <body>
 <div align="center">
 HTML
+    }
     _html("<span class=\"title\">$title</span><br />");
     _html("<br />");
     _html($self->_template_text('pagetitle2', %hash) . " " . $self->get_time());
@@ -276,10 +291,15 @@ $stats_gen<br />
 $author_text<br />
 $stats_text
 </span>
+HTML
+
+    if($self->{cfg}->{colorscheme} ne "none") {
+        print OUTPUT <<HTML;
 </div>
 </body>
 </html>
 HTML
+    }
 }
 
 sub _headline
@@ -410,6 +430,7 @@ sub _activetimes
 
         my $size = int(($self->{stats}->{times}{$hour} / $highest_value) * 100);
         my $percent = ($self->{stats}->{times}{$hour} / $self->{stats}->{parsedlines}) * 100;
+        my $lines_per_hour = $self->{stats}->{times}{$hour};
         $percent =~ s/(\.\d)\d+/$1/;
 
         if ($size < 1) {
@@ -421,7 +442,7 @@ sub _activetimes
         $image = "pic_v_".(int($hour/6)*6);
         $image = $self->{cfg}->{$image};
 
-        $output{$hour} = "<td align=\"center\" valign=\"bottom\" class=\"asmall\">$percent%<br /><img src=\"$self->{cfg}->{piclocation}/$image\" width=\"15\" height=\"$size\" alt=\"$percent\" /></td>\n";
+        $output{$hour} = "<td align=\"center\" valign=\"bottom\" class=\"asmall\">$percent%<br /><img src=\"$self->{cfg}->{piclocation}/$image\" width=\"15\" height=\"$size\" alt=\"$lines_per_hour\" title=\"$lines_per_hour\"/></td>\n";
     }
 
     _html("<table border=\"0\"><tr>\n");
@@ -504,7 +525,7 @@ sub _activenicks
     for (my $i = 0; $i < $self->{cfg}->{activenicks}; $i++) {
         my $c = $i + 1;
         my $nick = $active[$i];
-        my $visiblenick = $active[$i];
+        my $visiblenick;
 
         my $randomline;
         if (not defined $self->{stats}->{sayings}{$nick}) {
@@ -521,9 +542,11 @@ sub _activenicks
             $randomline = $self->_format_line($randomline);
         }
 
-        # Add a link to the nick if there is any
+        # Add a link to the nick if there is any and quote nick
         if ($self->{users}->{userlinks}{$nick}) {
-            $visiblenick = $self->_replace_links($self->{users}->{userlinks}{$nick}, $nick);
+            $visiblenick = $self->_format_word($self->{users}->{userlinks}{$nick}, $nick);
+        } else {
+            $visiblenick = $self->_format_word($nick);
         }
 
         my $color = $self->generate_colors($c);
@@ -669,7 +692,7 @@ sub _questions
 
     foreach my $nick (sort keys %{ $self->{stats}->{questions} }) {
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
-          if ($self->{stats}->{lines}{$nick} > 100) {
+          if ($self->{stats}->{lines}{$nick} > $self->{cfg}->{bignumbersthreshold}) {
               $qpercent{$nick} = ($self->{stats}->{questions}{$nick} / $self->{stats}->{lines}{$nick}) * 100;
               $qpercent{$nick} =~ s/(\.\d)\d+/$1/;
           }
@@ -711,7 +734,7 @@ sub _shoutpeople
 
     foreach my $nick (sort keys %{ $self->{stats}->{shouts} }) {
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
-          if ($self->{stats}->{lines}{$nick} > 100) {
+          if ($self->{stats}->{lines}{$nick} > $self->{cfg}->{bignumbersthreshold}) {
               $spercent{$nick} = ($self->{stats}->{shouts}{$nick} / $self->{stats}->{lines}{$nick}) * 100;
               $spercent{$nick} =~ s/(\.\d)\d+/$1/;
           }
@@ -755,7 +778,7 @@ sub _capspeople
 
     foreach my $nick (sort keys %{ $self->{stats}->{allcaps} }) {
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
-          if ($self->{stats}->{lines}{$nick} > 100) {
+          if ($self->{stats}->{lines}{$nick} > $self->{cfg}->{bignumbersthreshold}) {
               $cpercent{$nick} = $self->{stats}->{allcaps}{$nick} / $self->{stats}->{lines}{$nick} * 100;
               $cpercent{$nick} =~ s/(\.\d)\d+/$1/;
           }
@@ -1039,7 +1062,7 @@ sub _linelengths
 
     foreach my $nick (sort keys %{ $self->{stats}->{lengths} }) {
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
-          if ($self->{stats}->{lines}{$nick} > 100) {
+          if ($self->{stats}->{lines}{$nick} > $self->{cfg}->{bignumbersthreshold}) {
               $len{$nick} = $self->{stats}->{lengths}{$nick} / $self->{stats}->{lines}{$nick};
               $len{$nick} =~ s/(\.\d)\d+/$1/;
           }
@@ -1118,7 +1141,10 @@ sub _mostfoul
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
           if ($self->{stats}->{lines}{$nick} > 15) {
               $spercent{$nick} = $self->{stats}->{foul}{$nick} / $self->{stats}->{words}{$nick} * 100;
-              $spercent{$nick} =~ s/(\.\d)\d+/$1/;
+	      
+              my $dec = $self->{cfg}->{showfouldecimals};
+              $dec = 1 if($dec < 0); # default to 1
+              $spercent{$nick} =~ s/(\.\d{$dec})\d+/$1/;
           }
         }
     }
@@ -1169,7 +1195,7 @@ sub _mostsad
 
     foreach my $nick (sort keys %{ $self->{stats}->{frowns} }) {
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
-          if ($self->{stats}->{lines}{$nick} > 100) {
+          if ($self->{stats}->{lines}{$nick} > $self->{cfg}->{bignumbersthreshold}) {
               $spercent{$nick} = $self->{stats}->{frowns}{$nick} / $self->{stats}->{lines}{$nick} * 100;
               $spercent{$nick} =~ s/(\.\d)\d+/$1/;
           }
@@ -1450,7 +1476,7 @@ sub _mostsmiles
 
     foreach my $nick (sort keys %{ $self->{stats}->{smiles} }) {
         if ($self->{topactive}{$nick} || !$self->{cfg}->{showonlytop}) {
-          if ($self->{stats}->{lines}{$nick} > 100) {
+          if ($self->{stats}->{lines}{$nick} > $self->{cfg}->{bignumbersthreshold}) {
               $spercent{$nick} = $self->{stats}->{smiles}{$nick} / $self->{stats}->{lines}{$nick} * 100;
               $spercent{$nick} =~ s/(\.\d)\d+/$1/;
           }
@@ -1521,7 +1547,7 @@ sub _lasttopics
             my $hour = $self->{stats}->{topics}[$i]{hour};
             my $min  = $self->{stats}->{topics}[$i]{min};
 
-            $hash{nick} = $self->_format_word($nick);
+            $hash{nick} = $nick;
             $hash{time} = "$hour:$min";
             _html('<tr><td class="hicell"><i>' . $self->_format_line($topic) . '</i></td>');
             _html('<td class="hicell"><b>' . $self->_template_text('bylinetopic', %hash) . '</b></td></tr>');
@@ -1559,6 +1585,8 @@ sub _template_text
     }
 
     $hash{channel} = $self->{cfg}->{channel};
+    # the nick is sanitized here, everything else outside of _template_text
+    $hash{nick} = $self->_format_word($hash{nick}) if $hash{nick};
 
     foreach my $key (sort keys %hash) {
         $text =~ s/\[:$key\]/$hash{$key}/;
@@ -1577,10 +1605,10 @@ sub _template_text
 sub _format_word
 {
     # This function formats a word -- should ONLY be called on words used alone (EG: not whole line printed)
-    my ($self, $word) = @_;
+    my ($self, $word, $nick) = @_; # nick is only defined for user links in top table
 
     $word = htmlentities($word, $self->{cfg}->{charset});
-    $word = $self->_replace_links($word);
+    $word = $self->_replace_links($word, $nick);
     return $word;
 }
 
@@ -1755,9 +1783,9 @@ sub _mostreferencednicks
         for(my $i = 0; $i < $self->{cfg}->{nickhistory}; $i++) {
             last unless $i <= $#popular;
             my $a = $i + 1;
-            my $popular   = $popular[$i];
+            my $popular   = $self->_format_word($popular[$i]);
             my $wordcount = $self->{stats}->{wordcounts}{$popular[$i]};
-            my $lastused  = $self->{stats}->{wordnicks}{$popular[$i]};
+            my $lastused  = $self->_format_word($self->{stats}->{wordnicks}{$popular[$i]});
 
             my $class;
             if ($a == 1) {
@@ -1795,12 +1823,14 @@ sub _mosturls
         for(my $i = 0; $i < $self->{cfg}->{urlhistory}; $i++) {
             last unless $i < @sorturls;
             my $a = $i + 1;
-            my $sorturl  = $sorturls[$i];
             my $urlcount = $self->{stats}->{urlcounts}{$sorturls[$i]};
             my $lastused = $self->{stats}->{urlnicks}{$sorturls[$i]};
-            if (length($sorturl) > 60) {
-                $sorturl = substr($sorturl, 0, 60);
+            my $printurl = $sorturls[$i];
+            if (length($printurl) > 60) {
+                $printurl = substr($printurl, 0, 60);
             }
+            $printurl = htmlentities($printurl, $self->{cfg}->{charset});
+            my $linkurl = urlencode($sorturls[$i]);
             my $class;
             if ($a == 1) {
                 $class = 'hirankc';
@@ -1808,7 +1838,7 @@ sub _mosturls
                 $class = 'rankc';
             }
             _html("<tr><td class=\"$class\">$a</td>");
-            _html("<td class=\"hicell\"><a href=\"$sorturls[$i]\">$sorturl</a></td>");
+            _html("<td class=\"hicell\"><a href=\"$linkurl\">$printurl</a></td>");
             _html("<td class=\"hicell\">$urlcount</td>");
             _html("<td class=\"hicell\">$lastused</td>");
             _html("</tr>");
@@ -1830,28 +1860,40 @@ sub _legend
 }
 
 
+sub _replace_url # used internally by _replace_links
+{
+    my ($self, $url, $www, $ftp, $text) = @_;
+    $url = "http://$url" if $www;
+    $url = "ftp://$url" if $ftp;
+    my $texturl = $self->_template_text("newwindow");
+    return "<a href=\"$url\" target=\"_blank\" title=\"$texturl $url\">" . $self->_split_long_text($text) . '</a>';
+}
+
+
+sub _replace_email # used internally by _replace_links
+{
+    my ($self, $mailto, $user, $domain, $nick) = @_;
+    $mailto = '' if $nick or not $mailto;
+    $nick ||= "$user&#64;$domain"; # obfuscate mail address
+    my $textmail = $self->_template_text("mailto");
+    return "<a href=\"mailto:$user&#64;$domain\" title=\"$textmail $nick\">" . $self->_split_long_text($mailto . $nick) . "<\/a>";
+}
+
+
 sub _replace_links
 {
-    # Sub to replace urls and e-mail addys to links
-    my ($self, $str, $nick) = @_;
+    # replace URLs and email addresses by links
+    my ($self, $str, $nick) = @_; # nick is only defined for user links in top table
 
     # Regular expressions are taken from match_urls() and match_email() in
     # Common.pm
 
-    my $texturl = $self->_template_text("newwindow");
-    my $textmail = $self->_template_text("mailto");
     my (@str) = split(/ /,$str);
 
     foreach (@str) {
-        if (m/(http|https|ftp|telnet|news)(:\/\/[-a-zA-Z0-9_\/~:@]+\.[-a-zA-Z0-9.,_~=:&amp;@%?#\/+]+)/o) {
-            my $nick = $nick || $1 . $2;
-            $_ = "<a href=\"$1$2\" target=\"_blank\" title=\"$texturl $1$2\">" . $self->_split_long_text($nick) . '</a>';
-        } elsif (m/(^|[^:])\b([-a-zA-Z0-9._]+@[-a-zA-Z0-9_]+\.[-a-zA-Z0-9._]+)/o) {
-            my $nick = $nick || $1 . $2;
-            $_ = "$1<a href=\"mailto:$2\" title=\"$textmail $nick\">" . $self->_split_long_text($nick) . "<\/a>";
-        } else {
-            $_ = $self->_split_long_text($_);
-        }
+        s/((?:(?:https?|ftp|telnet|news):\/\/|(?:(?:(www)|(ftp))[\w-]*\.))[-\w\/~\@:]+\.\S+[\w\/])/$self->_replace_url($1, $2, $3, $nick || $1)/egio
+            or s/(mailto:)?([-\w.]+)@([-\w]+\.[-\w.]+)/$self->_replace_email($1, $2, $3, $nick)/egio
+            or $_ = $self->_split_long_text($_);
      }
 
     return join(' ', @str);
@@ -1891,9 +1933,10 @@ sub _user_wordtimes
     my $nick  = shift;
     my $top   = shift;
 
-    my $bar      = "";
-    my $len      = ($self->{stats}->{words}{$nick} / $self->{stats}->{words}{$top}) * 100;
+    $self->{stats}->{words}{$nick} ||= 0;
+    my $len = ($self->{stats}->{words}{$nick} / $self->{stats}->{words}{$top}) * 100;
 
+    my $bar = "";
     for (my $i = 0; $i <= 3; $i++) {
         next if not defined $self->{stats}->{word_times}{$nick}[$i];
         my $w = int(($self->{stats}->{word_times}{$nick}[$i] / $self->{stats}->{words}{$nick}) * $len);
@@ -1935,24 +1978,27 @@ sub _user_pic
     return unless $self->{users}->{userpics}{$nick} or $self->{cfg}->{defaultpic};
 
     my $rowspan = $self->{cfg}->{userpics} ? " rowspan=\"$self->{cfg}->{userpics}\"" : "";
-    _html("<td style=\"background-color: $color\" align=\"center\" valign=\"middle\"$rowspan>");
+    my $output = "<td style=\"background-color: $color\" align=\"center\" valign=\"middle\"$rowspan>";
 
     my $biguserpic = $self->{users}->{biguserpics}{$nick};
-    $biguserpic = $self->{cfg}->{imagepath} . randomglob($biguserpic, $self->{cfg}->{imageglobpath})
-        if $biguserpic and $biguserpic !~ /^http:\/\//i;
-    $biguserpic = "<a href=\"$biguserpic\">" if $biguserpic;
-    _html($biguserpic) if $biguserpic;
+    if ($biguserpic) {
+        $biguserpic = $self->{cfg}->{imagepath} .
+            randomglob($biguserpic, $self->{cfg}->{imageglobpath})
+                if $biguserpic !~ /^http:\/\//i;
+        $output .= "<a href=\"$biguserpic\">";
+    }
 
     my $pic = $self->{users}->{userpics}{$nick} || $self->{cfg}->{defaultpic};
     $pic = $self->{cfg}->{imagepath} . randomglob($pic, $self->{cfg}->{imageglobpath})
         unless $pic =~ /^http:\/\//i;
     my $height = $self->{cfg}->{picheight} ? " height=\"$self->{cfg}->{picheight}\"" : "";
     my $width = $self->{cfg}->{picwidth} ? " width=\"$self->{cfg}->{picwidth}\"" : "";
-    my $alt = $self->{users}->{userpics}{$nick} ? " alt=\"$nick\"" : "";
-    _html("<img src=\"$pic\"$width$height$alt />");
+    my $alt = $self->{users}->{userpics}{$nick} ? " alt=\"$nick\" title=\"$nick\"" : "";
+    my $border = $biguserpic ? ' border="0"' : '';
+    $output .= "<img src=\"$pic\"$width$height$alt$border />";
 
-    _html("</a>") if $biguserpic;
-    _html("</td>");
+    $output .= "</a>" if $biguserpic;
+    _html("$output</td>");
 }
 
 sub _mostnicks
@@ -1975,27 +2021,28 @@ sub _mostnicks
         _html("<td>&nbsp;</td><td class=\"tdtop\"><b>$nick_txt</b></td>");
         _html("<td class=\"tdtop\"><b>$names_txt</b></td></tr>");
 
-        for(my $i = 0; $i < $self->{cfg}->{mostnickshistory}; $i++) {
-            next if is_ignored($sortnicks[$i]);
-            last unless $i < @sortnicks;
-            my $nickcount = keys %{ $self->{stats}->{nicks}->{$sortnicks[$i]} };
-            my $nickused = join(", ", values %{ $self->{stats}->{nicks}->{$sortnicks[$i]} });
+        my $a = 0;
+        foreach my $nick (@sortnicks) {
+            next if is_ignored($nick);
+            my $nickcount = keys %{ $self->{stats}->{nicks}->{$nick} };
+            my $nickused = join(", ", values %{ $self->{stats}->{nicks}->{$nick} });
 
-            next unless ($nickcount > 1);
+            last unless ($nickcount > 1);
 
-            my $a = $i + 1;
+            $a++;
             my $class = $a == 1 ? 'hirankc' : 'rankc';
             my $n = $nickcount > 1 ? $names1 : $names2;
 
             _html("<tr><td class=\"$class\">$a</td>");
             if ($self->{cfg}->{mostnicksverbose}) { 
-                _html("<td class=\"hicell\">$sortnicks[$i] ($nickcount $n)</td>");
+                _html("<td class=\"hicell\">$nick ($nickcount $n)</td>");
                 _html("<td class=\"hicell\" valign='top'>$nickused</td>");
             } else {
-                _html("<td class=\"hicell\">$sortnicks[$i]</td>");
+                _html("<td class=\"hicell\">$nick</td>");
                 _html("<td class=\"hicell\" valign='top'>$nickcount $n</td>");
             }
             _html("</tr>");
+            last if $a >= $self->{cfg}->{mostnickshistory};
         }
         _html("</table>");
     }
@@ -2074,7 +2121,7 @@ sub _mostactivebyhour
                             my $w = int(($count / $maxlines) * 100) || 1;
                             _html("<img src=\"$self->{cfg}->{piclocation}/$self->{cfg}->{$pic}\" border=\"0\" width=\"$w\" height=\"15\" align=\"middle\" alt=\"\" />");
                         }
-                        _html($nick." - ".$count);
+                        _html($self->_format_word($nick)." - ".$count);
                         _html("</td>");
                     } else {
                         _html("<td class=\"hicell\">&nbsp;</td>");
@@ -2112,6 +2159,7 @@ sub _topactive {
     }
             
     (@top_active) = @top_active[0..($self->{cfg}->{activenicks}+$self->{cfg}->{activenicks2}-1)];
+    $self->{stats}->{topactive_lines} = @top_active ? $self->{stats}->{lines}{$top_active[0]} : 1;
             
     foreach (@top_active) {
         $self->{topactive}{$_} = 1;
