@@ -30,7 +30,8 @@ sub new
     $self->{parser} = $self->_choose_format($self->{cfg}->{format});
 
     if($self->{cfg}->{logcharsetfallback} and not $self->{cfg}->{logcharset}) {
-        print STDERR "Warning: LogCharset undefined, assuming LogCharset = LogCharsetFallback.\n";
+        print "LogCharset undefined, assuming LogCharset = LogCharsetFallback\n"
+            unless ($self->{cfg}->{silent});
         $self->{cfg}->{logcharset} = $self->{cfg}->{logcharsetfallback};
     }
 
@@ -46,10 +47,16 @@ sub new
                 $self->{iconvfallback} = Text::Iconv->new($self->{cfg}->{logcharsetfallback}, $self->{cfg}->{charset});
             }
         } else {
-            print STDERR "Text::Iconv is not available, skipping charset conversion of logfiles\n";
+            print "Text::Iconv is not installed, skipping charset conversion of logfiles\n"
+                unless ($self->{cfg}->{silent});
         }
     }
 
+    # precompile the regexps used (we can't use /o since the config might be different per channel)
+    $self->{foulwords_regexp} = qr/($self->{cfg}->{foulwords})/i;
+    $self->{ignorewords_regexp} = qr/$self->{cfg}->{ignorewords}/i;
+    $self->{violentwords_regexp} = qr/^($self->{cfg}->{violentwords}) (\S+)(.*)/i;
+
     return $self;
 }
 
@@ -113,7 +120,7 @@ sub analyze
         $stats{processtime}{mins} = sprintf('%02d', $min);
         $stats{processtime}{secs} = sprintf('%02d', $sec);
 
-        print "Channel analyzed succesfully in $processtime on ",
+        print "Channel analyzed successfully in $processtime on ",
         scalar localtime(time()), "\n"
             unless ($self->{cfg}->{silent});
 
@@ -231,7 +238,7 @@ sub _parse_dir
 
         if($self->{cfg}->{nfiles} > 0) {
             my $shift = @filesarray - $self->{cfg}->{nfiles};
-            splice(@filesarray, 0, $shift);
+            splice(@filesarray, 0, $shift) if $shift > 0;
         }
 
         foreach my $file (@filesarray) {
@@ -247,7 +254,7 @@ sub _parse_file
     my $self = shift;
     my ($stats, $lines, $file, $state) = @_;
 
-    print "Analyzing log($file) in '$self->{cfg}->{format}' format...\n"
+    print "Analyzing log $file in '$self->{cfg}->{format}' format...\n"
         unless ($self->{cfg}->{silent});
 
     if ($file =~ /.bz2?$/ && -f $file) {
@@ -275,7 +282,8 @@ sub _parse_file
             if($line2) {
                 $line = $line2;
             } else {
-                print STDERR "Charset conversion failed for '$line'\n";
+                print "Charset conversion failed for '$line'\n"
+                    unless ($self->{cfg}->{silent});
             }
         }
 
@@ -362,7 +370,7 @@ sub _parse_file
                         }
                     }
 
-                    if (my @foul = $saying =~ /($self->{cfg}->{foulwords})/io) {
+                    if (my @foul = $saying =~ /($self->{foulwords_regexp})/) {
                         $stats->{foul}{$nick} += scalar @foul;
                         push @{ $lines->{foullines}{$nick} }, $line;
                     }
@@ -382,14 +390,13 @@ sub _parse_file
                     if (my @urls = match_urls($saying)) {
                         foreach my $url (@urls) {
                             if(!url_is_ignored($url)) {
-                                $url =~ s/&/&amp;/g;
                                 $stats->{urlcounts}{$url}++;
                                 $stats->{urlnicks}{$url} = $nick;
                             }
                         }
                     }
 
-                    _parse_words($stats, $saying, $nick, $self->{cfg}->{ignorewords}, $hour);
+                    _parse_words($stats, $saying, $nick, $self->{ignorewords_regexp}, $hour);
                 }
             }
             $lastnormal = $line;
@@ -430,7 +437,7 @@ sub _parse_file
                 $stats->{lastvisited}{$nick} = $stats->{days};
                 $stats->{line_times}{$nick}[int($hour/6)]++;
 
-                if ($saying =~ /^($self->{cfg}->{violentwords}) (\S+)(.*)/o) {
+                if ($saying =~ /$self->{violentwords_regexp}/) {
                     my $victim;
                     unless ($victim = is_nick($2)) {
                         foreach my $trynick (split(/\s+/, $3)) {
@@ -451,7 +458,7 @@ sub _parse_file
 
                 $stats->{lengths}{$nick} += length($saying);
 
-                _parse_words($stats, $saying, $nick, $self->{cfg}->{ignorewords}, $hour);
+                _parse_words($stats, $saying, $nick, $self->{ignorewords_regexp}, $hour);
             }
         }
 
@@ -522,7 +529,7 @@ sub _parse_file
 
     close(LOGFILE);
 
-    print "Finished analyzing log, $stats->{days} days total.\n"
+    print "Finished analyzing log, $stats->{days} days, $stats->{parsedlines} lines total\n"
         unless ($self->{cfg}->{silent});
 }
 
@@ -590,8 +597,14 @@ sub _parse_words
         # Also ignore stuff from URLs.
         next if ($word =~ m/^https?$|^\/\//o);
 
-        $stats->{wordcounts}{$word}++;
-        $stats->{wordnicks}{$word} = $nick;
+	# uniquify nicks
+	if (my $realnick = is_nick($word)) {
+            $stats->{wordcounts}{$realnick}++;
+            $stats->{wordnicks}{$realnick} = $nick;
+	} else {
+            $stats->{wordcounts}{$word}++;
+            $stats->{wordnicks}{$word} = $nick;
+	}
     }
 }
 
@@ -610,7 +623,7 @@ sub _random_line
 {
     my ($self, $stats, $lines, $key, $nick, $count) = @_;
     my $random = ${ $lines->{$key}{$nick} }[rand@{ $lines->{$key}{$nick} }];
-    if ($self->{cfg}->{noignoredquotes} && $random =~ /$self->{cfg}->{ignorewords}/io) {
+    if ($self->{cfg}->{noignoredquotes} && $random =~ /$self->{ignorewords_regexp}/i) {
         return '' if ($count > 20);
         return $self->_random_line($stats, $lines, $key, $nick, ++$count);
     } else {
@@ -626,7 +639,6 @@ sub _uniquify_nicks {
 	    # The lc() is an attempt at being case insensitive.
 	    if (lc($realnick) ne lc($word)) {
 	        $stats->{wordcounts}{$realnick} += $stats->{wordcounts}{$word};
-	        $stats->{wordnicks}{$realnick}   = $stats->{wordnicks}{$word};
 	        delete $stats->{wordcounts}{$word};
 	        delete $stats->{wordnicks}{$word};
 	    }
